# coding=utf-8
from .monkeyLib import *
from .selectXml import GetCoordinate

from . import mod_logger as logger


class CheckFuncStuff(object):

    def __init__(self, devices, pname,maina):
        self.devices = devices
        self.package_name = pname
        self.main_a = maina

    def check_app_is_autohome(self):
        """
        通过判断当前运行的activity判断app是否为汽车之家
        :return: True:是  False:不是
        """
        activityname = getActivityName(self.devices, self.package_name)
        if activityname.find(self.package_name) < 0:
            # 运行包不是本包的话，重启应用
            restartAutohome(self.devices, self.package_name, self.main_a)
            self.check_app_is_autohome()
        else:
            return True

    def check_ui_xml_location_is_change(self, old_loc):
        """
        判断页面是否有变动，通过location点去判断
        :return:  True: 页面未变化 False:页面变化
        """
        saveUi(self.devices)
        logger.info("判断页面变动前，先dump xml文件取点")
        # 将ui布局放到服务器中
        fname = "".join(str(time.time()).split(".")) + ".xml"
        # path = os.path.split(os.path.realpath(__file__))[0] + "/../data/" + fname
        path = fname
        # 上传过程是否出错，如果出错，则重启app
        if saveUiToFile(self.devices, path):
            pass
        else:
            restartAutohome(self.devices, self.package_name, self.main_a)
            return False
        # 读取UI中的xy
        sx = GetCoordinate(path)
        new_loc = sx.get_seleced_location()
        os.remove(path=path)
        logger.info("删除本地不用的xml文件。")
        if old_loc == new_loc:
            return True
        else:
            return False

    def check_activity_is_change(self, old_act):
        """
        检测activity是否有变动
        :return: True: activity无变化 False:有变化
        """
        act = getActivityName(self.devices, self.package_name)
        logger.info(act)
        if act == old_act:
            return True
        else:
            return False

    def check_real_activity_is_change(self, old_act):
        act = getActivityName(self.devices, self.package_name)
        real_old_act = old_act.strip().split(" ")[1]
        real_act = act.strip().split(" ")[1]
        if real_act == real_old_act:
            return True
        else:
            return False

    def check_now_activity_in_input(self, act):
        """
        检测当前activity是否在传入的activity中
        :param act: 传入的activity
        :return: True: 在， False： 不在
        """
        now_act = getActivityName(self.devices, self.package_name)
        real_now_act = now_act.strip().split(" ")[1]
        if act in real_now_act:
            return True
        else:
            return False


if __name__ == '__main__':
    # logger.info(CheckFuncStuff("1ee481f2", "b").check_ui_xml_location_is_change(1))
    # c = CheckFuncStuff()
    pass


# coding=utf-8
import subprocess
import time

import os
from .monkeyLib import get_aapt_command, get_adb_command

# from lib.mod_db import DataBase
from . import mod_logger as logger


class GetAndroidInfo(object):
    def __init__(self, devices):
        self.devices = devices

    def run_comm(self, comm):
        """执行comm"""
        try:
            p = subprocess.Popen(
                comm,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                shell=True,
                start_new_session=True
            )
            return p
        except Exception as e:
            logger.info(e)

    def get_cpu(self, pname):
        comm = "%s -s %s shell top -n 1" % (get_adb_command(), self.devices)
        # todo cpu获取方式不同机型仍有问题 部分手机cpu数据会超100%，如：10.168.136.11:5555
        p = self.run_comm(comm)
        try:
            for x in p.stdout.readlines():
                x = str(x, encoding="utf-8")
                if pname[:10] in x:
                    a = x.split(" ")
                    while '' in a:
                        a.remove("")
                    logger.info("cpu: %s" % a)
                    return a[-4]
        except Exception as e:
            logger.error("获取CPU失败：%s" % e)
            return False

    def get_mem(self, pname):
        comm = "%s -s %s shell dumpsys meminfo %s -d" % (get_adb_command(),
        self.devices, pname)
        p = self.run_comm(comm)
        try:
            for x in p.stdout.readlines():
                x = str(x, encoding="utf-8")
                if "TOTAL" in x:
                    x = x.strip()
                    a = x.split("   ")
                    return a[1]
        except Exception as e:
            logger.error("获取内存信息失败：%s" % e)
            return False

    def write_cpu_and_mem(self, activity, t, pname):
        """
        写主activity cpu mem
        :param main_a:
        :return:
        """
        activity = activity.strip().split(" ")[1]
        if pname in activity:
            cpu, mem = self.get_cpu(pname), self.get_mem(pname)
            if cpu and mem:
                # db = DataBase()
                # sql = 'INSERT cloudperformence SET action = "%s", cpu = "%s", ' \
                #       'mem = "%s", activity = "%s", report_id = %d' % (t, cpu, mem, activity, sqlid)
                logger.info("cpu: %s, mem: %s" % (cpu, mem))
                # db.update(sql)
                # db.close()
            else:
                logger.error("CPU或内存获取失败，此次不做存储操作。")
                logger.error("CPU:%s, MEM:%s" % (str(cpu), str(mem)))
        else:
            logger.info("pname not in activity\n")
            logger.info("pname is:%s\nactivity is:%s" % (pname, activity))

    def get_error(self, path, pname):
        comm = "%s -s %s logcat -d \*:E | grep %s " % (get_adb_command(),
            self.devices, pname)  # .split(".")[2])
        p = self.run_comm(comm)
        result = p.stdout.read()
        result = self.transfer_content(str(result, "utf-8"))
        if len(result) != 0:
            if len(result) > 3600:
                result = result[:3599]
            logger.error(result)
            # name = self.get_time() + ".png"
            # self.get_screenshot(path, name)
        else:
            pass

    # 判断日志中是否存在ANR: ANRManager
    def get_anr(self, path):
        comm = "%s -s %s logcat -v time -d | grep ANR" % (get_adb_command(),self.devices)
        p = self.run_comm(comm)
        result = p.stdout.read()
        result = self.transfer_content(str(result, "utf-8"))
        if len(result) != 0:
            if len(result) > 3600:
                result = result[:3599]
            logger.error(result)
            # name = self.get_time() + ".png"
            # self.get_screenshot(path, name)
        else:
            pass

    def transfer_content(self, content):
        if content is None:
            return None
        else:
            string = ""
            for c in content:
                if c == '"':
                    string += '\\\"'
                elif c == "'":
                    string += "\\\'"
                elif c == "\\":
                    string += "\\\\"
                else:
                    string += c
            return string

    def get_screenshot(self, path, name):
        """
        获取屏幕截图
        :return:
        """
        # path = os.path.split(os.path.realpath(__file__))[0] + "/../log/%s/%s.png" % (self.devices, name)
        n = "%s/%s" % (path, name)
        os.system(
            "%s -s %s shell screencap /sdcard/screenshot.png" % (get_adb_command(), self.devices))
        os.system(
            "%s -s %s pull /sdcard/screenshot.png %s" % (get_adb_command(), self.devices, n))

    def get_package_name_and_activity(self, fpath):
        """
        获取包名和主Activity
        :param fpath: apk路径
        :return: 1：包名 2：主Activity包
        """
        # db = DataBase()
        # res = db.fetch_all('select * from cloudreportlist WHERE id=%d' % sqlid)

        # apkname = res[0].get("apkname")
        comm = '%s d badging "%s"' % (get_aapt_command(), fpath)
        logger.info(comm)
        p = self.run_comm(comm)
        result = p.stdout.readlines()
        for x in result:
            x = str(x, encoding="utf-8")
            if "package" in x:
                package_line = x
            if "launchable-activity" in x:
                main_a = x
                break
        try:
            package_name = package_line.split("'")[1]
            main_name = main_a.split("'")[1]
            # db.update('update cloudreportlist set packagename = "%s", mainactname = "%s" WHERE id = %d' % (
            #     package_name, main_name, sqlid))
            # db.close()
            return package_name, main_name
        except Exception as e:
            # db.close()
            assert e


    def get_time(self):
        """
        get time
        :return:
        """
        t = time.time()
        nt = "".join(str(t).split("."))
        return nt


if __name__ == '__main__':
    # logger.info(os.path.split(os.path.realpath(__file__))[0])
    # logger.info("获取com.cubic.autohome的内存使用量及CPU占用率")
    device = '1ee481f2'  # xiaomi 5
    # # device = "FA6AB0311937" # piexl
    g = GetAndroidInfo(device)
    # logger.info(g.get_cpu())
    # logger.info(g.get_mem())
    logger.info(g.get_cpu())



from functools import wraps

def except_this(errors=(Exception, ), default_value="default"):


    def decorator_func(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except errors as e:
                print("error func:", func.__name__)
                print("ERROR: ", repr(e))
                return default_value

        return wrapper

    return decorator_func


@except_this()
def example(a):
    b = int(a)
    print(b)
    return b


if __name__ == '__main__':
    example("a")


import sys

from .base import MonkeyRunner

from .mod_logger import Logger as logger


def get_argvs():
    if len(sys.argv) == 5:
        devices = sys.argv[1]
        times = sys.argv[2]
        schame = sys.argv[3]
        activity = sys.argv[4]
        return devices, times, schame, activity
    else:
        print("args is not full")
        return False


def run(device="", fpath="", scheme="", activity="", d=0, h=0, m=5, s=0):
    """
    run the monkey tast
    :param device: the devices of android
    :param fpath: the path of apk package
    :param scheme: the scheme of your business line
    :param activity: the activity of you scheme
    :param d: day
    :param h: hour
    :param m: minus
    :param s: second
    :return: None
    """
    if device == "" or fpath == "" or scheme == "" or activity == "":
        logger.error("缺少参数")
    else:
        run = MonkeyRunner(devices=device, fpath=fpath, activity=activity, schame=scheme)
        run.begin(d=d, h=h, m=m, s=s)


if __name__ == '__main__':
    devices = "4b94aa19"
    fpath = "/home/hanz/workspace/AndroidTest/apk/autohome_1540628242.apk"
    schame = "autohome://article/videodetail?newsid=64772"
    activity = "com.autohome.main.article.activitys.ArticlePagerActivity"
    run(device=devices, fpath=fpath, scheme=schame, activity=activity, m=5)
    # from lib.checkfunc import CheckFuncStuff
    # c = CheckFuncStuff(devices=devices, pname="com.cubic.autohome", maina="df")
    # c.check_activity_is_change("asdf")

    # from urllib import parse
    # def cache_key():
    #     args = ([('id', '88'), ('token', 'asdfa')])
    #     key = "/report" + '?' + parse.urlencode([
    #         (k, v) for k in sorted(args) for v in sorted(args.getlist(k))
    #     ])
    #     return key



# coding=utf-8
try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET
import json

import re
from .base_decorator import except_this

from . import mod_logger as logger


class GetCoordinate(object):

    def __init__(self, path):
        self.path = path

    def analysis_xml(self):
        """
        解析xml文件
        :return:
        """
        try:
            tree = ET.ElementTree(file=self.path)
            data = tree.getroot()
            d_list = []
            for child_of_data in data.iter(tag="node"):
                child_json = json.loads(json.dumps(child_of_data.attrib))
                yield child_json
        except Exception as e:
            logger.error("解析UI.xml文件报错！\n %s" % e)
            return []

    @except_this()
    def get_all_location(self):
        """
        获取可点击坐标点
        :return:
        """
        data = self.analysis_xml()
        data_list = []
        if data != []:
            for x in data:
                bounds = x.get("bounds")
                pattern = re.compile(r"\d+")
                coord = pattern.findall(bounds)
                Xpoint = (int(coord[2]) - int(coord[0])) / 2.0 + int(coord[0])
                Ypoint = (int(coord[3]) - int(coord[1])) / 2.0 + int(coord[1])
                location_dict = {
                    'x': Xpoint,
                    'y': Ypoint
                }
                data_list.append(location_dict)
        else:
            pass
        return data_list

    @except_this()
    def get_seleced_location(self):
        """
        获取指定条件的坐标点
        :return:
        """
        data = self.analysis_xml()
        data_list = []
        # 排除的class name
        # todo
        exclude_class_name = [
            # 'android.widget.FrameLayout',
            # 'android.widget.LinearLayout',
            # 'android.widget.RelativeLayout'
        ]
        if data != []:
            for x in data:
                classname = str(x.get("class"))
                enabled = x['enabled']
                clickable = x['clickable']
                focusable = x['focusable']
                if classname not in exclude_class_name:
                    if (enabled == "true" and clickable == "true") or (enabled == True and focusable == True): # and focusable == "true"
                        bounds = x.get("bounds")
                        pattern = re.compile(r"\d+")
                        coord = pattern.findall(bounds)
                        Xpoint = (int(coord[2]) - int(coord[0])) / 2.0 + int(coord[0])
                        Ypoint = (int(coord[3]) - int(coord[1])) / 2.0 + int(coord[1])
                        location_dict = {
                            'x': Xpoint,
                            'y': Ypoint
                        }
                        data_list.append(location_dict)
                    else:
                        pass
                else:
                    pass
        else:
            pass
        return data_list


if __name__ == '__main__':
    g = GetCoordinate("../data/ui.xml")
    c = g.analysis_xml()
    logger.info(c)
    s = g.get_seleced_location()
    print(s)



data = [{"action": "swipe", "activity": "com.cubic.autohome/.MainActivity", "cpu": "20%", "id": 554, "mem": "199029"},
        {"action": "click", "activity": "com.cubic.autohome/com.autohome.plugin.search.ui.activity.SearchActivity",
         "cpu": "10%", "id": 555, "mem": "240133"},
        {"action": "swipe", "activity": "com.cubic.autohome/com.autohome.plugin.search.ui.activity.SearchActivity",
         "cpu": "12%", "id": 556, "mem": "246731"},
        {"action": "click", "activity": "com.cubic.autohome/com.autohome.plugin.search.ui.activity.SearchActivity",
         "cpu": "19%", "id": 557, "mem": "294868"},
        {"action": "swipe", "activity": "com.cubic.autohome/com.autohome.plugin.search.ui.activity.SearchActivity",
         "cpu": "21%", "id": 558, "mem": "299532"},
        {"action": "click", "activity": "com.cubic.autohome/com.autohome.main.car.activitys.SpecMainActivity",
         "cpu": "17%", "id": 559, "mem": "288430"},
        {"action": "swipe", "activity": "com.cubic.autohome/com.autohome.main.car.activitys.SpecMainActivity",
         "cpu": "15%", "id": 560, "mem": "295339"},
        {"action": "swipe", "activity": "com.cubic.autohome/.MainActivity", "cpu": "13%", "id": 561, "mem": "162021"},
        {"action": "click", "activity": "com.cubic.autohome/.MainActivity", "cpu": "8%", "id": 562, "mem": "164035"},
        {"action": "swipe", "activity": "com.cubic.autohome/.MainActivity", "cpu": "29%", "id": 563, "mem": "208471"},
        {"action": "swipe", "activity": "com.cubic.autohome/.MainActivity", "cpu": "14%", "id": 593, "mem": "174652"},
        {"action": "click", "activity": "com.cubic.autohome/.MainActivity", "cpu": "13%", "id": 594, "mem": "170896"},
        {"action": "swipe", "activity": "com.cubic.autohome/.MainActivity", "cpu": "11%", "id": 595, "mem": "173476"},
        {"action": "click", "activity": "com.cubic.autohome/.MainActivity", "cpu": "11%", "id": 596, "mem": "147820"},
        {"action": "swipe", "activity": "com.cubic.autohome/.MainActivity", "cpu": "7%", "id": 597, "mem": "213040"},
        {"action": "click", "activity": "com.cubic.autohome/.MainActivity", "cpu": "8%", "id": 598, "mem": "214707"},
        {"action": "swipe", "activity": "com.cubic.autohome/.MainActivity", "cpu": "6%", "id": 599, "mem": "215968"},
        {"action": "click", "activity": "com.cubic.autohome/.MainActivity", "cpu": "5%", "id": 600, "mem": "275720"},
        ]


def dis_repeat(data):
    from collections import Counter
    c = Counter()
    for ch in [x.get("activity") for x in data]:
        c[ch] = c[ch] + 1
    return c


def join_result(d):
    bbb = []
    cpu = 0
    mem = 0
    for x, y in d.items():
        aaa = {}
        for z in data:
            if z.get("activity") == x:
                cpu += int(z.get("cpu").split("%")[0])
                mem += int(z.get("mem"))
        aaa["activity"] = x
        aaa["cpu"] = "%d%%" % int(cpu / y)
        aaa["mem"] = "%dKB" % int(mem / y)
        bbb.append(aaa)
    return bbb


# b = dis_repeat(data)
# logger.info(join_result(b))

def check(func):
    def wrapper():
        print("this is check ")
        return func()

    return wrapper


@check
def foo():
    print("this is foo")


from monkeyrun.monkeyLib import getActivityName, restart_autohome_by_scheme

activity = "com.autohome.main.article.activitys.ArticlePagerActivity"
schame = "autohome://article/videodetail?newsid=64772"


def check_activity(a):
    """
    检测activity是否为传入的，如果不是，则重启
    :param func: 
    :return: 
    """

    def decorator(func):

        def wrapper(*args, **kwargs):
            now_act = getActivityName("9929c6e1", "com.cubic.autohome")
            print(now_act)
            real_now_act = now_act.strip().split(" ")[1]
            print(a)
            print(real_now_act)
            if a not in real_now_act:
                restart_autohome_by_scheme("9929c6e1", "com.cubic.autohome", schame)
                return
            else:
                print(real_now_act)
                print(args)
                pass
            return func(*args, **kwargs)

        return wrapper

    return decorator


class Foo(object):
    def __init__(self, func):
        self._func = func

    def __call__(self, *args, **kwargs):
        print("this is ")
        self._func
        print("hahaha")

    # @check_activity(activity)
    # @Foo
    # def monkeySwipeRight(device, x, y):
    #     """向右滑动"""
    #     print("adfasdf")
    #     os.system("%s -s %s shell input swipe  %s %s %s %s" % (get_adb_command(),
    #                                                            device, x * 0.2, y * 0.5, x * 0.8, y * 0.5))
    #     print("end")


    # monkeySwipeRight("9929c6e1", 200, 400)

    # if __name__ == '__main__':
    #     monkeySwipeRight("9929c6e1", 200, 400)
    # foo("wanghan")


from monkeyrun.base_decorator import except_this


@except_this()
def test1(abc):
    tt = int(abc)
    print(tt)
    return tt


result_data = {
    "kartun": {"cpuType": "", "kartunList": []},
    "ipaIncrease": 10307.3,
    "processId": "test43",
    "crash": "",
    "compileHistId": "test0001",
    "VCDuration": {"CHViewController": 229},
    "bgNetWork": {"requestUrl": []},
    "version": "8.7.0",
    "pluginDuration": {"starts": {"汽车助手插件": 10}, "registers": {}},
    "memoryMax": {"ctime": "2017-12-14 18:52:00", "memoryValue": 104.620002746582}
}


if __name__ == '__main__':
    test1("sdfa")



# coding=utf-8
import subprocess
import time

import os
import random

from . import mod_logger as logger


def uninstall_apk(device, pname):
    os.system('%s -s %s uninstall %s' % (get_adb_command(), device, pname))


def install_apk(device, path):
    os.system('%s -s %s install %s' % (get_adb_command(), device, path))


def monkeyTap(device, x, y):
    os.system("%s -s %s shell input tap %s %s" % (get_adb_command(), device, x, y))
    logger.info("点击")


def monkeyGoBack(device):
    os.system("%s -s %s shell input keyevent 4" % (get_adb_command(), device))


def saveUi(device):
    os.system("%s -s %s shell uiautomator dump /sdcard/ui.xml" % (get_adb_command(), device))


# def _mkdirs():
#     if not os.path.exists(log_dir):
#         try:
#             os.makedirs(log_dir)
#         except Exception as e:
#             print(str(e))


def saveUiToFile(device, path):
    comm = "%s -s %s pull /sdcard/ui.xml %s" % (get_adb_command(), device, path)
    p = run_comm(comm)
    result = p.stdout.readlines()
    for x in result:
        x = str(x, encoding="utf-8")
        if "error" in x:
            return False
    os.system("%s -s %s shell rm -rf /sdcard/ui.xml" % (get_adb_command(), device))
    return True


def getActivityName(device, pname):
    p = os.popen(
        "%s -s %s shell dumpsys activity top | grep ACTIVITY" % (get_adb_command(), device))
    for x in p.readlines():
        if pname in x:
            return x
    return ""


def startAutohome(device, pname, maina):
    # os.system("%s -s %s shell am start -n %s/%s " % (get_adb_command(), device, pname, maina))
    comm = "%s -s %s shell am start -n %s/%s " % (get_adb_command(), device, pname, maina)
    p = run_comm(comm)
    result = p.stderr.readlines()
    for x in result:
        x = str(x, encoding="utf-8")
        if "Warning: Activity not started, its current task has been brought to the front" in x:
            monkeyGoBack(device)
    time.sleep(10)


def closeActivity(device, pname):
    os.system("%s -s %s shell am force-stop %s" % (get_adb_command(), device, pname))
    time.sleep(2)


def restartAutohome(device, pname, maina):
    comm = "%s -s %s shell am start -S %s/%s " % (get_adb_command(), device, pname, maina)
    p = run_comm(comm)
    result = p.stderr.readlines()
    for x in result:
        x = str(x, encoding="utf-8")
        if "Warning: Activity not started, its current task has been brought to the front" in x:
            monkeyGoBack(device)
    time.sleep(5)


def start_autohome_by_schema(device, schema):
    # os.system("%s  shell am start -n com.cubic.autohome/.LogoActivity ")
    os.system(
        "%s -s %s shell am start -a android.intent.action.VIEW -d %s" % (
            get_adb_command(), device, schema))
    time.sleep(8)


def restart_autohome_by_scheme(device, pname, scheme):
    closeActivity(device, pname)
    start_autohome_by_schema(device, scheme)


def monkeySwipeBottom(device, x, y):
    """向下滑动"""
    os.system("%s -s %s  shell input swipe %s %s %s %s" % (get_adb_command(),
                                                           device, x * 0.5, y * 0.2, x * 0.5, y * 0.8))


def monkeySwipeUp(device, x, y):
    """向上滑动"""
    os.system("%s -s %s  shell input swipe  %s %s %s %s" % (get_adb_command(),
                                                            device, x * 0.5, y * 0.8, x * 0.5, y * 0.2))


def monkeySwipeLeft(device, x, y):
    """向左滑动"""
    os.system("%s -s %s shell input swipe  %s %s %s %s" % (get_adb_command(),
                                                           device, x * 0.8, y * 0.5, x * 0.2, y * 0.5))


def monkeySwipeRight(device, x, y):
    """向右滑动"""
    os.system("%s -s %s shell input swipe  %s %s %s %s" % (
        get_adb_command(), device, x * 0.2, y * 0.5, x * 0.8, y * 0.5)
              )


def del_log(device):
    comm = "%s -s %s logcat -c" % (get_adb_command(), device)
    os.system(comm)


def getDisplaySize(device):
    p = os.popen("%s -s %s shell  wm size" % (get_adb_command(), device))
    temp = p.read().replace('Physical size:', '').strip().replace('\n', '')
    temp = temp.split('x')
    displaysize = {
        'w': temp[0],
        'h': temp[1]
    }
    return displaysize


def randomMonkey(device):
    displaysize = getDisplaySize(device)
    randomInt = random.randint(1, 3)
    if randomInt == 1:
        # 下滑操作，加大幅度
        monkeySwipeBottom(device, int(displaysize['w']), int(displaysize['h']))
        logger.info("下滑一次")
    elif randomInt == 2:
        monkeySwipeUp(device, int(displaysize['w']), int(displaysize['h']))
        monkeySwipeUp(device, int(displaysize['w']), int(displaysize['h']))
        monkeySwipeUp(device, int(displaysize['w']), int(displaysize['h']))
        logger.info("上滑三次")
    elif randomInt == 3:
        monkeySwipeLeft(device, int(displaysize['w']), int(displaysize['h']))
        monkeySwipeLeft(device, int(displaysize['w']), int(displaysize['h']))
        monkeySwipeLeft(device, int(displaysize['w']), int(displaysize['h']))
        logger.info("左滑三次")
        # elif randomInt == 4:
        #     monkeySwipeRight(device, int(displaysize['w']), int(displaysize['h']))
        #     logger.info("右滑一次")


def test_upload(devices, path):
    comm = "%s -s %s pull /sdcard/ui.xml %s" % (get_adb_command(), devices, path)
    p = run_comm(comm)
    result = p.stderr.readlines()
    for x in result:
        x = str(x, encoding="utf-8")
        if "error" in x:
            return False
    return True


def run_comm(comm):
    """执行comm"""
    try:
        p = subprocess.Popen(
            comm,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=True,
            start_new_session=True
        )
        return p
    except Exception as e:
        logger.error("执行失败：%s\n 命令：%s" % (e, comm))


def get_adb_command():
    comm = 'whereis adb'
    p = run_comm(comm)
    data = str(p.stdout.read(), encoding="utf-8")
    result = data.split(" ")[1].strip()
    return result


def get_aapt_command():
    comm = 'whereis aapt'
    p = run_comm(comm)
    data = str(p.stdout.read(), encoding="utf-8")
    result = data.split(" ")[1].strip()
    return result


if __name__ == '__main__':
    os.system(
        "/home/hanz/programs/android-sdk-linux/platform-tools/adb shell am start -a android.intent.action.VIEW -d autohome://article/videodetail?newsid=64772")
    # print("11", get_adb_command(), "22")
    # print("---")
    # print(get_aapt_command())
    # path = os.path.split(os.path.realpath(__file__))[0] + "/../data/"
    # logger.info(path)
    # saveUi("1ee481f2")
    # test_upload("1ee481f2", path)



# -*- coding: utf-8 -*-
"""
例程：
from somelib import logger

# 默认log存放目录,需要在程序入口调用才能生效,可省略
logger.log_dir = "./app"
# log文件名前缀,需要在程序入口调用才能生效,可省略
logger.log_name = "test_log"

conf = logger.Logger()
conf.debug('debug')
conf.warn('tr-warn')
conf.info('ds-info')
conf.error('ss-error')

"""

import os, time, threading
import datetime
import logging
import logging.handlers

try:
    import codecs
except ImportError:
    codecs = None

log_dir = "log"
log_name = "applog"

_logger_init_lock = threading.Lock()


class MyHandler(logging.handlers.TimedRotatingFileHandler):
    """
    自己定义的TimedRotatingFileHandler
    """

    def __init__(self, log_dir, file_name_prefix):
        self.log_dir = log_dir
        self.file_name_prefix = file_name_prefix

        self._mkdirs()

        self.baseFilename = "%s.%s.log" % (os.path.join(self.log_dir, file_name_prefix),
                                           time.strftime("%Y%m%d"))

        logging.handlers.TimedRotatingFileHandler.__init__(self,
                                                           self.baseFilename,
                                                           when='midnight', interval=1,
                                                           backupCount=0, encoding=None)

    def doRollover(self):
        self.stream.close()
        # get the time that this sequence started at and make it a TimeTuple
        t = self.rolloverAt - self.interval
        timeTuple = time.localtime(t)
        self.baseFilename = "%s.%s.log" % (os.path.join(self.log_dir, self.file_name_prefix),
                                           time.strftime("%Y%m%d"))
        if self.encoding:
            self.stream = codecs.open(self.baseFilename, 'a', self.encoding)
        else:
            self.stream = open(self.baseFilename, 'a')
        self.rolloverAt = self.rolloverAt + self.interval

    def _mkdirs(self):
        if not os.path.exists(self.log_dir):
            try:
                os.makedirs(self.log_dir)
            except Exception as e:
                print(str(e))


class Logger(object):
    __instance = None

    def __new__(classtype, *args, **kwargs):
        _logger_init_lock.acquire()
        if classtype != type(classtype.__instance):
            classtype.__instance = object.__new__(classtype, *args, **kwargs)
            classtype.__instance.init()

        _logger_init_lock.release()
        return classtype.__instance

    def init(self):
        # 创建日志目录
        global log_dir, log_name
        self.log_dir = log_dir
        self.log_name = log_name

        self.is_debug = True
        self.is_info = True
        self.is_warn = True
        self.is_error = True

        self.logger_formatter = "[%(asctime)-15s,%(levelname)s] %(message)s"
        self.file_formatter = "[%(asctime)-15s,%(levelname)s] %(message)s"
        self._initLogger()

    def _initLogger(self):
        # 初始化logger
        logging.basicConfig(format=self.logger_formatter)
        self.logger = logging.getLogger("_sys")
        self.logger.setLevel(logging.DEBUG)

        # info、warn、error都放到info文件
        # error单独放到error文件
        for t in (("info", logging.INFO),
                  ("error", logging.ERROR)):
            filehandler = MyHandler(self.log_dir,
                                    "%s.%s" % (self.log_name, t[0]))
            filehandler.suffix = "%Y%m%d.log"
            filehandler.setLevel(t[1])
            filehandler.setFormatter(logging.Formatter(self.file_formatter))
            self.logger.addHandler(filehandler)

            # debug 单独放到debug文件
        filehandler = MyHandler(self.log_dir,
                                "%s.debug" % self.log_name)
        filehandler.suffix = "%Y%m%d.log"
        filehandler.setLevel(logging.DEBUG)
        filehandler.setFormatter(logging.Formatter(self.file_formatter))
        self.logger.addHandler(filehandler)

    def getLogger(self):
        return self.logger

    def debug(self, msg):
        if self.is_debug:
            self.logger.debug(msg)

    def info(self, msg):
        if self.is_info:
            self.logger.info(msg)

    def warn(self, msg):
        if self.is_warn:
            self.logger.warning(msg)

    def error(self, msg):
        if self.is_error:
            self.logger.error(msg)


def info(msg):
    Logger().info(msg)


def warn(msg):
    Logger().warn(msg)


def debug(msg):
    Logger().debug(msg)


def error(msg):
    Logger().error(msg)



# coding=utf-8
import datetime
import string
import time

from .checkfunc import CheckFuncStuff
from .getinfo import GetAndroidInfo
from .monkeyLib import *
from .selectXml import GetCoordinate

# from lib.mod_db import DataBase
from . import mod_logger as logger


class MonkeyRunner(object):
    def __init__(self, devices, fpath=None, schame=None, activity=None):
        self.devices = devices
        self.schame = schame
        self.activity = activity
        self.fpath = fpath
        self.get_info = GetAndroidInfo(self.devices)
        self.imgpath = os.path.split(os.path.abspath(__file__))[0] + "/image"
        self.apkpath = os.path.split(os.path.abspath(__file__))[0] + "/apk"
        self.pname, self.main_a = self.get_info.get_package_name_and_activity(fpath)
        logger.info("packagename: %s, mainactivity: %s" % (self.pname, self.main_a))
        self.checkf = CheckFuncStuff(self.devices, self.pname, self.main_a)

    def mkdir(self):
        """
        初始化创建文件夹
        :return:
        """
        today = time.strftime("%Y%m%d", time.localtime())
        path = os.path.split(os.path.realpath(__file__))[0] + "/log/" + self.devices + \
               "/" + today + "/" + "".join(self.pname.split(".")) + "_" + self.random_letter(4)
        logger.info(path)
        folder = os.path.exists(path)
        if not folder:
            os.makedirs(path)
        return path

    def random_letter(self, n):
        # 生成随机字段串 大小写+数字
        letter = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(n))
        return letter

    def endTime(self, days, hours, minutes, seconds):
        """
        计算结束时间
        :param hours: 时长，单位小时
        :return: 具体时间
        """
        now_time = datetime.datetime.now()
        return now_time + datetime.timedelta(days=days, hours=hours, minutes=minutes, seconds=seconds)

    def dump_and_get_location(self):
        """
        dump ui.xml并采集坐标点数据
        :return: location
        """
        # 保存UI布局到手机
        saveUi(self.devices)
        logger.info("获取坐标点前，先dump xml文件")
        # 将ui布局放到服务器中
        fname = "".join(str(time.time()).split(".")) + ".xml"
        # path = os.path.split(os.path.realpath(__file__))[0] + "/data/" + fname
        path = fname
        logger.info("保存文件路径：%s" % path)
        if saveUiToFile(self.devices, path):
            pass
        else:
            # restartAutohome(self.devices, self.pname, self.main_a)
            restart_autohome_by_scheme(self.devices, self.pname, self.schame)
            return self.dump_and_get_location()
        # 读取UI中的xy
        sx = GetCoordinate(path)
        s_location = sx.get_seleced_location()
        os.remove(path=path)
        logger.info("删除本地不用的xml文件。")
        return s_location

    def random_monkey_swipe_test(self):
        """
        执行五次随机操作
        :return:
        """
        del_log(self.devices)
        for x in range(5):
            logger.info("滑动之前清空logcat")
            time.sleep(0.5)
            randomMonkey(self.devices)
        logger.info("滑动完成后记录一下，cpu，内存，是否有错误日志，是否有ANR")
        self.get_info.write_cpu_and_mem(getActivityName(self.devices, self.pname), "swipe", self.pname)
        self.get_info.get_anr(self.imgpath)
        self.get_info.get_error(self.imgpath, self.pname)

    def random_monkey_click_test(self, location):
        """
        执行随机点击操作
        :return:
        """
        # location = self.dump_and_get_location()
        # 选点
        if location == []:
            monkeyGoBack(self.devices)
            logger.info("坐标点为空，点击无效，返回上一层。")
            return
        else:
            choice_loc = random.choice(location)
        logger.info("点击之前清空logcat")
        del_log(self.devices)
        monkeyTap(self.devices, str(choice_loc['x']), str(choice_loc['y']))
        if self.checkf.check_ui_xml_location_is_change(location):
            logger.info("页面未变化，去掉刚才选的点，再次执行随机点击操作")
            location.remove(choice_loc)
            self.random_monkey_click_test(location)
        else:
            # 页面变化后，点击完成后记录一下，cpu，内存，是否有错误日志，是否有ANR
            self.get_info.write_cpu_and_mem(getActivityName(self.devices, self.pname), "click", self.pname)
            self.get_info.get_anr(self.imgpath)
            self.get_info.get_error(self.imgpath, self.pname)

    def begin(self, d=0, h=0, m=5, s=0):
        """
        执行性能测试
        :param h: 执行时长，单位：小时
        :return: 无
        """
        # 卸载安装
        try:
            # uninstall_apk(self.devices, self.pname)
            # apk_name = self.fpath
            # logger.info(apk_name)
            # install_apk(self.devices, apk_name)
            # 打印开始时间
            endtime = self.endTime(days=d, hours=h, minutes=m, seconds=s)
            start_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            logger.info(start_time)
            logger.info("start time: %s" % str(time.ctime()))
            # 循环
            seed, wee, count = 0, 0, 0
            # startAutohome(self.devices, self.pname, self.main_a)
            restart_autohome_by_scheme(self.devices, self.pname, self.schame)
            while datetime.datetime.now() < endtime:
                # 每执行5次，检测是一下是否为本应用
                wee += 1
                if wee > 5:
                    self.checkf.check_app_is_autohome()
                    wee = 0
                    logger.info("每执行5次，检测是一下是否为本应用")
                logger.info("判断是否autohome")
                old_act = getActivityName(self.devices, self.pname)
                self.checkf.check_app_is_autohome()
                logger.info("执行页面滑动操作")
                self.random_monkey_swipe_test()
                logger.info("执行页面随机点击操作")
                location = self.dump_and_get_location()
                self.random_monkey_click_test(location)
                logger.info("判断是否还在activity中，如果不在，则重启scheme")
                if self.checkf.check_now_activity_in_input(self.activity):
                    pass
                else:
                    restart_autohome_by_scheme(self.devices, self.pname, self.schame)
                    continue
                if self.checkf.check_activity_is_change(old_act):
                    # activity无变化，加1
                    seed += 1
                    logger.info(seed)
                    if seed > 4:
                        monkeyGoBack(self.devices)
                        time.sleep(0.5)
                        logger.info("同一activity执行5次，执行goback")
                        # again_act = getActivityName(self.devices)
                        if self.checkf.check_real_activity_is_change(old_act):
                            # restartAutohome(self.devices, self.pname, self.main_a)
                            restart_autohome_by_scheme(self.devices, self.pname, self.schame)
                            seed = 0
                else:
                    # activity有变化，重置seed
                    seed = 0
                # 循环一次加1
                count += 1
            closeActivity(self.devices, self.pname)
            # 打印结束时间
            end_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            logger.info("执行顺利完成，共执行 %d 次" % count)
            logger.info("end time: %s" % str(time.ctime()))
            # uninstall_apk(self.devices, self.pname)
        except Exception as e:
            logger.error("流程中间失败：%s" % e)
            end_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            logger.error("执行中间出错，置数据库状态成功。")
            logger.error("end time: %s" % str(time.ctime()))
            # uninstall_apk(self.devices, self.pname)

# devices = "1ee481f2"
# while True:
#     devices = get_device()
#     if devices == "":
#         logger.info("无空闲设备，等待300S。。。")
#         time.sleep(300)
#     else:
#         run = MonkeyRunner(devices)
#         run.run(d=0, h=1, m=0, s=0)
#         break
